Package: prodlim
Title: Product-Limit Estimation for Censored Event History Analysis
Version: 2026.03.11
Authors@R: person(given = c("Thomas", "A."),
                    family = "Gerds",
                    role = c("aut", "cre"),
                    email = "tag@biostat.ku.dk")
Description: Fast and user friendly implementation of nonparametric estimators
    for censored event history (survival) analysis. Kaplan-Meier and
    Aalen-Johansen method.
Depends: R (>= 4.1.0)
Imports: Rcpp (>= 0.11.5), stats, rlang, data.table, grDevices,
        ggplot2, scales, graphics, diagram, survival, KernSmooth, lava
Suggests: tibble, ggthemes, riskRegression, testthat, etm, survey
LinkingTo: Rcpp
Maintainer: Thomas A. Gerds <tag@biostat.ku.dk>
BugReports: https://github.com/tagteam/prodlim/issues
License: GPL (>= 2)
Packaged: 2026-03-11 12:23:03 UTC; tag
RoxygenNote: 7.3.2
NeedsCompilation: yes
Author: Thomas A. Gerds [aut, cre]
Repository: CRAN
Date/Publication: 2026-03-11 17:30:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-28 13:39:00 UTC; unix
Archs: prodlim.so.dSYM
