# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,EventHistory.frame)
S3method(as.data.frame,prodlim)
S3method(as.data.table,prodlim)
S3method(getStates,Hist)
S3method(getStates,prodlim)
S3method(lifeTab,competing.risks)
S3method(lifeTab,survival)
S3method(plot,Hist)
S3method(plot,prodlim)
S3method(predict,prodlim)
S3method(print,Hist)
S3method(print,IntIndex)
S3method(print,neighborhood)
S3method(print,potential.followup)
S3method(print,prodlim)
S3method(print,quantile.prodlim)
S3method(print,summary.prodlim)
S3method(quantile,prodlim)
S3method(summary,Hist)
S3method(summary,prodlim)
export('[.Hist')
export(EventHistory.frame)
export(Hist)
export(List2Matrix)
export(PercentAxis)
export(SimCompRisk)
export(SimSurv)
export(SmartControl)
export(as.data.frame.prodlim)
export(as.data.table.prodlim)
export(as_tibble.prodlim)
export(atRisk)
export(backGround)
export(checkCauses)
export(confInt)
export(crModel)
export(dimColor)
export(followup)
export(geom_prodlim)
export(getEvent)
export(getStates)
export(ggprodlim)
export(head.Hist)
export(is.na.Hist)
export(jackknife)
export(jackknife.competing.risks)
export(jackknife.survival)
export(leaveOneOut)
export(leaveOneOut.competing.risks)
export(leaveOneOut.survival)
export(lines.prodlim)
export(markTime)
export(meanNeighbors)
export(model.design)
export(neighborhood)
export(parseFormula)
export(parseSpecialNames)
export(plot.Hist)
export(plot.prodlim)
export(plotCompetingRiskModel)
export(plotIllnessDeathModel)
export(predictSurvIndividual)
export(prodlim)
export(quantile.prodlim)
export(redist)
export(row.match)
export(sindex)
export(stopTime)
export(str.Hist)
export(strip.terms)
export(subset.Hist)
export(summary.prodlim)
export(survModel)
export(tail.Hist)
import(data.table)
import(lava)
importFrom(Rcpp,sourceCpp)
importFrom(grDevices,rainbow)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,grconvertX)
importFrom(graphics,grconvertY)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,rect)
importFrom(graphics,segments)
importFrom(graphics,strheight)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(stats,.getXlevels)
importFrom(stats,delete.response)
importFrom(stats,drop.terms)
importFrom(stats,formula)
importFrom(stats,get_all_vars)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,na.omit)
importFrom(stats,pchisq)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,reformulate)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(stats,update.formula)
importFrom(survival,Surv)
importFrom(survival,cluster)
importFrom(survival,survdiff)
useDynLib(prodlim, .registration=TRUE)
