R/`prodlim`: Product limit method for event history analysis including the Kaplan-Meier and the Aalen-Johansen estimator
========================================================================================================
[![CRAN\_Status\_Badge](https://www.r-pkg.org/badges/version/prodlim)](https://CRAN.R-project.org/package=prodlim) [![](https://cranlogs.r-pkg.org/badges/prodlim)](https://CRAN.R-project.org/package=prodlim) 
[![R-CMD-check](https://github.com/tagteam/prodlim/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/tagteam/prodlim/actions/workflows/R-CMD-check.yaml)

